(* $Novell: schema.c,v 1.15 2003/05/12 13:01:45 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
*   schema.c
***************************************************************************
*   Description:  schema.c demonstrates the following:
*                  - Fetching all schema element definitions
*                  - Reading and printing schema element definitions
*                  - Deleting schema element definitions
*                  - Adding (Extending) new schema element definitions
*
****************************************************************************)
program schema;

uses sysutils, ldap;

(**********************************************************************
    print_schema:

    Reads schema element definitions from the local copy of schema,
    according to the type passed in.  print_schema iterates through
    definitions using ldap_schema_get_by_index. (If only one element
    is needed ldap_schema_get_by_name will lookup the element by name.)

    Three nested loops are used:
    1) Loop through all schema element definitions
    2) Loop through the field names for an element
    3) Loop through the field values for an element                 *)

procedure print_schema(ldschema: pLDAPSchema; schema_type: integer);
var
    index,              //* iterates through schema elements     */
    field,              //* iterates through fields of a element */
    value,              //* iterates through values of a field   */
    count,              //* count of schema elements             */
    rc: integer;        //* LDAP result code                     */

    element: pLDAPSchemaElement;
    names  : ppchar8;    //* holds the field names       */
    ppn    : ppchar8;
    values : ppchar8;    //* holds the values of a field */
    ppv    : ppchar8;
begin
    //* Iterate through the definitions */
    count := ldap_schema_get_count(ldschema, schema_type);
    for index:=0 to count -1 do begin
        //* Retrieve the schema element definition */
        rc := ldap_schema_get_by_index(ldschema, index, schema_type, element);
        if (rc <> LDAP_SUCCESS) then begin
            writeln('Error in function ldap_schema_get_by_index: ',ldap_err2string(rc));
            break;
        end;

        //* Retrieve defined field names */
        rc := ldap_schema_get_field_names(element, names);
        if (rc <> LDAP_SUCCESS) then begin
            writeln('Error in function ldap_schema_get_field_names: ',ldap_err2string(rc));
            break;
        end;

        //* Iterate through each field name*/
        field := 0;
        ppn := names;
        while (ppn^<>nil) do begin
            writeln(,ppn^);

            //* Retrieve values of the field */
            rc := ldap_schema_get_field_values(element, ppn^, values);

            if (rc <> LDAP_SUCCESS) then begin
                writeln('Error in function ldap_schema_get_by_index: ',
                         ldap_err2string(rc));
                break;
            end;

            //* Iterate through each value */
            ppv := values;
            if (ppv^ <> nil) then begin
               write('  ',ppv^);
               inc(ppv);
            end;
            while (ppv^<>nil) do begin
               write(', ',ppv^);
               inc(ppv);
            end{for};
            writeln;
            ldap_value_free(values);
            inc(ppn);
        end{while};
        ldap_value_free(names);
    end{for};
end; //-------------------------------------------------------------------------

const
    timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */
var
    ldapHost   : AnsiString;
    loginDN    : AnsiString;
    password   : AnsiString;
    ld         : hLDAP;

    rc, ldapPort, version: int32;

    ldschema   : pLDAPSchema; // := NIL

    //* Variables for adding a new addribute */
//    LDAPSchemaMod m_oid, m_name, m_desc, m_syntax, m_x, *mod[6];
    m_oid, m_name, m_desc, m_syntax, m_x : LDAPSchemaMod;
    s_mod: array[0..5] of pLDAPSchemaMod;

    //* Syntax for a Case Ignore String */
    syntax : array[0..1] of pchar8;
    oid    : array[0..1] of pchar8;
    name   : array[0..1] of pchar8;
    desc   : array[0..1] of pchar8;
//    char *syntax[]  = {"1.3.6.1.4.1.1466.115.121.1.15", NULL};
//    char *oid []    = {"2.16.840.1.113719.1.187.4.1", NULL};
//    char *name[]    = {"TestAttribute", NULL};
//    char *desc[]    = {"Test attribute definition", NULL};

    ///* This field specifies that the attribute doesn't syncronize
    //   immediately.  Any X- Qualifier can be used */
    x_field  : pchar8;
    x_values : array[0..1] of pchar8;
//    char *x_field   = "X-NDS_NOT_SCHED_SYNC_IMMEDIATE";
//    char *x_values[]= {"1", NULL };
begin
    syntax[0]   := '1.3.6.1.4.1.1466.115.121.1.15';
    syntax[1]   := nil;
    oid[0]      := '2.16.840.1.113719.1.187.4.1';
    oid[1]      := nil;
    name[0]     := 'TestAttribute';
    name[1]     := nil;
    desc[0]     := 'Test attribute definition';
    desc[1]     := nil;
    x_field     := 'X-NDS_NOT_SCHED_SYNC_IMMEDIATE';
    x_values[0] := '1';
    x_values[1] := nil;

    if (paramcount()<>4) then begin
      writeln('Usage:   Usage: schema <host name> <port number> <login dn> <password>');
      writeln('    Example: schema Acme.com 389 cn=admin,o=Acme secret');
      halt(1);
    end;

    ldapHost     := paramstr(1);
    ldapPort     := StrToInt(paramstr(2));
    loginDN      := paramstr(3);
    password     := paramstr(4);

    ///* Set LDAP version to 3, needed to read root DSE.
    // * Also set connection timeout.
    // */
    version := LDAP_VERSION3;
    writeln('ldap_set_option');
    ldap_set_option(NIL, LDAP_OPT_PROTOCOL_VERSION, version);
    ldap_set_option(NIL, LDAP_OPT_NETWORK_TIMEOUT, timeOut);

    //* open an LDAP session */
    writeln('ldap_init');
    ld := ldap_init(pchar8(ldapHost),ldapPort);

    if (ld = nil) then begin
      writeln('LDAP session initialization failed');
      halt(1);
    end;
    writeln('LDAP session initialized');

    //* bind */
    writeln('ldap_simple_bind_s');
    rc := ldap_simple_bind_s(ld, pchar8(loginDN), pchar8(password));
    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldap_simple_bind_s: ', ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;
    writeln('Bind successful.');

    //* Fetch all schema element definitions for the directory */
    writeln('ldap_schema_fetch');
    rc := ldap_schema_fetch(ld, ldschema, nil);
    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldap_schema_fetch: ', ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;

    //* Reading and printing schema element definitions: */
    print_schema(ldschema, LDAP_SCHEMA_ATTRIBUTE_TYPE);
    print_schema(ldschema, LDAP_SCHEMA_OBJECT_CLASS);
    print_schema(ldschema, LDAP_SCHEMA_MATCHING_RULE);
    print_schema(ldschema, LDAP_SCHEMA_MATCHING_RULE_USE);
    print_schema(ldschema, LDAP_SCHEMA_SYNTAX);
    print_schema(ldschema, LDAP_SCHEMA_DIT_CONTENT_RULE);
    print_schema(ldschema, LDAP_SCHEMA_DIT_STRUCTURE_RULE);


    //* Delete the definition if it already exists */
    writeln('ldap_schema_delete');
    rc := ldap_schema_delete(ldschema, LDAP_SCHEMA_ATTRIBUTE_TYPE, name[0]);

    //* If the definition already existed, commit the deletion */
    if (rc = LDAP_SUCCESS) then begin
        rc := ldap_schema_save(ld, ldschema, nil);
        if (rc <> LDAP_SUCCESS) then begin
            writeln('Could not remove an existing attribute: ', ldap_err2string(rc));
        end;
    end;


    ///*  Setting up add schema element definition:
    //    The 'op' field of LDAP_SCHEMA_MOD is not used by
    //    ldap_schema_add.
    // */
    m_oid.fieldName    := LDAP_SCHEMA_OID;
    m_oid.values       := addr(oid);
    s_mod[0]           := addr(m_oid);

    m_name.fieldName   := LDAP_SCHEMA_NAMES;
    m_name.values      := addr(name);
    s_mod[1]           := addr(m_name);

    m_desc.fieldName   := LDAP_SCHEMA_DESCRIPTION;
    m_desc.values      := addr(desc);
    s_mod[2]           := addr(m_desc);

    m_syntax.fieldName := LDAP_SCHEMA_SYNTAX_OID;
    m_syntax.values    := addr(syntax);
    s_mod[3]           := addr(m_syntax);

    m_x.fieldName      := x_field;
    m_x.values         := addr(x_values);
    s_mod[4]           := addr(m_x);

    s_mod[5]           := nil;

    // Add new definition to the directory */
    writeln('ldap_schema_add');
    rc := ldap_schema_add(ldschema, LDAP_SCHEMA_ATTRIBUTE_TYPE, addr(s_mod));
    if (rc <> LDAP_SUCCESS) then begin
      writeln('Error in function ldap_schema_add: ', ldap_err2string(rc));
    end;

    writeln('ldap_schema_save');
    rc := ldap_schema_save(ld, ldschema, nil);
    if (rc <> LDAP_SUCCESS) then begin
      writeln('Error in function ldap_schema_save: ', ldap_err2string(rc));
    end;

    ldap_schema_free(ldschema);
    ldap_unbind(ld);
end.


